%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% HMM-EEG analysis                                                        %
%                                                                         %
% This file belongs to the following paper:                               %
% The Discovery of Processing Stages:                                     %
% Analyzing EEG data with Hidden Markov Models.                           %
% Borst, J.P. & Anderson, J.R.                                            %%                                                                         %
%                                                                         %
% Jelmer Borst (j.p.borst@rug.nl)                                         %
% John Anderson                                                           %
% 140723                                                                  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Instructions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% This file contains matlab code to run the main analyses in the paper. It
% is organized based on the figures in the paper, and recreates those
% figures. The analysis for each figure can be run independently - 
% intermediate results are provided.

% The analyses were performed on two different computers:
%   Machine 1: Intel Xeon E5, 16 CPUs @ 2.20 GHz, 128 GB memory, 
%              12 cores used for parallel processing in Matlab. 
%   Machine 2: Intel Core 2 Duo, 2.66 GHz, 8GB memory,
%              no parallel processing in Matlab.
% We will provide the runtime of the main parts of the analysis for those
% two machines.

% For plotting Fig 5A and Fig 7 eeglab is required; we used version 11 but
% later versions might also work. See http://sccn.ucsd.edu/eeglab/.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% General

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%set the working directory to the location of this file.
%cd('/path/to/HMM_EEG/');




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Load data (required for Figure 4, Analysis Fig 5-7, Figure 7)
% Data should be downloaded separately)
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('hmm_data.mat');

% This loads:

% data_base_multi: 236007x644 matrix that contains the normalized data,
%                  in snapshot format. The first 4 columns contain
%                  subject number, trial number, sample-in-trial, and
%                  condition information, respectively. The other 640
%                  columns contain the data. Each row contain 80 ms worth
%                  of data, or one snapshot. Each set of 32 columns
%                  represents 4 ms, where the column indicates the
%                  electrode in the following order: FP1, FPZ, FP2, F7, F3,
%                  FZ, F4, F8, FT7, FC3, FCZ, FC4, FT8, T7, C3, CZ, C4, T8,
%                  TP7, CP3, CPZ, CP4, TP8, P7, P3, PZ, P4, P8, POZ, O1,
%                  OZ, and O2.

% mu_base_multi_1, sigma_base_multi_1: normalization means and standard
%                                      deviations for data_base_multi. That
%                                      is, data_base_multi(:,5) *
%                                      sigma_base_multi_1(1) + 
%                                      mu_base_multi(1) gives the original 
%                                      data.

% data_base_multi_pca: like data_base_multi, but PCA of 640 data columns,
%                      and normalized after PCA.

% coeff_base_multi: normalized coefficients of PCA, which means that
%                   data_base_multi_pca(:,5:end) * coeff_base_multi' ==
%                   data_base_multi(:,5:end).

% Select data for analyses:
conditions = data_base_multi_pca(:,4);

% select targets/re-paired foils:
sel = conditions < 10;

act = data_base_multi_pca(sel,[3 5:end]); %sample-in-trial + data
conds = conditions(sel,:);
conds = conds - 1; %conditions have to start at 1
trials = data_base_multi_pca(sel,2);
subs = data_base_multi_pca(sel,1);

% select new foils:
sel = conditions > 9;

actNF = data_base_multi_pca(sel,[3 5:end]); %sample-in-trial + data
condsNF = conditions(sel,:);
condsNF = condsNF - 9; %conditions have to start at 1
trialsNF = data_base_multi_pca(sel,2);
subsNF = data_base_multi_pca(sel,1);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure 4: LOOCV to determine the optimal number of states
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%run LOOCV for targets/re-paired foils:

%input:
%act:       NxM matrix with the first column sample in trial, other columns PCA
%           components
%subs:      column vector with N rows, indicating subject
%conds:     column vector with N rows, indicating condition
%10:        max number of states to try. Note that it goes much faster with
%           fewer states.
%trials:    column vector with N rows, indicating trial

%output:
%likebest:  loglikelihoods, subject x state matrix
%params:    estimated gamma parameters, 1 x state cellstring,
%           each cell is a state x 2 gamma params x 8
%           conditions x 20 subjects matrix
%means:     state signatures, 1 x state cellstring, each cell
%           is a 20 pca weights x state x 20 subjects matrix

matlabpool %for parallel processing

[likebest params means] = generalLinearCondsTimes2(act(:,1:(100+1)), subs, conds, 10,trials); %100+1 because we used 100 PCA components
save('loocv_targets_repaired_foils.mat', 'likebest', 'params', 'means');

matlabpool close

%Machine 1: 55 hours and 2 minutes.
%Machine 2: too long, at least 23 days, probably more.


%run LOOCV for new foils:

matlabpool %for parallel processing

tic
[likebestNF paramsNF meansNF] = generalLinearCondsTimes2(actNF(:,1:(100+1)), subsNF, condsNF, 10,trialsNF);
toc

save('loocv_new_foils.mat', 'likebestNF', 'paramsNF', 'meansNF');

matlabpool close

%Machine 1: 3 hours and 41 minutes
%Machine 2: 35 hours and 4 minutes



%make figure 4

%load data
load('loocv_targets_repaired_foils.mat');
load('loocv_new_foils.mat');

%average and substract state 1
likebest = mean(likebest);
likebest = likebest - likebest(1);
likebestNF = mean(likebestNF);
likebestNF = likebestNF - likebestNF(1);

%plot
figure;
axes('box', 'on', 'xlim', [.5 10.5], 'ylim', [-75 2000]);
set(gca,'FontSize',14);
set(gca,'FontName','Helvetica');
xlabel('Number of States','FontSize',15);
ylabel('Loglikelihood Gain over 1 State','FontSize',15);
set(gcf,'Color',[1 1 1]);

line(1:10,likebest,'color', 'b','linestyle', '-','marker','o','linewidth',1.0,'MarkerSize',10)
line(1:10,likebestNF,'color', 'r','linestyle', '--','marker','s','linewidth',1.0,'MarkerSize',10)
legend('Targets/re-paired foils', 'New foils', 'location','southeast')
legend('boxoff')



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Analysis figs 5-7: calculate state durations per subject for 6 state 
% solution for targets/re-paired foils and 5 state solution for new foils.
% Requires loading data, see above.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Targets / re-paired foils

%input:
%act:       NxM matrix with the first column sample in trial, other columns PCA
%           components
%subs:      column vector with N rows, indicating subject
%conds:     column vector with N rows, indicating condition
%n_states:  the number of states

%output:
%durations:     conditions x state matrix of state durations of overall solution 
%meanSDs:       conditions x state matrix with mean SDs over subjects 
%params:        estimated gamma params per subject. states x 2 x 8 conditions x
%               20 subjects matrix
%signature:     brain signature of overall solution: 20 component weights x 6
%               states matrix
%paramstore:    gamma parameters of overall solution
%probpairs:     N x 6 states matrix containing the probability that each sample
%               is in the states

n_states = 6; %6 states

matlabpool

[durations, meanSDs, params, signature, paramstore, probpairs] = calcSDs(act(:,1:101),conds,subs, n_states);
save('solution_6_targets_repaired_foils.mat', 'durations', 'meanSDs', 'params', 'signature', 'paramstore', 'probpairs');

matlabpool close

%Machine 1: 55 minutes
%Machine 2: 2 hours and 9 minutes



% New foils

n_states = 5; %5 states

matlabpool

[durationsNF, meanSDsNF, paramsNF, signatureNF, paramstoreNF, probpairsNF] = calcSDs(actNF(:,1:101),condsNF,subsNF, n_states);
save('solution_5_new_foils.mat', 'durationsNF', 'meanSDsNF', 'paramsNF', 'signatureNF', 'paramstoreNF', 'probpairsNF');

matlabpool close
    
%Machine 1: 3 minutes
%Machine 2: 9 minutes





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure 5: Signatures + gamma distributions
% Requires loading data, see above.
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('solution_6_targets_repaired_foils.mat');
load('solution_5_new_foils.mat');
load('chanlocs.mat'); %channel locations



% Brain signatures

%go back from PCA to original
sig_ori = signature' * coeff_base_multi(:,1:100)';
sig_oriNF = signatureNF' * coeff_base_multi(:,1:100)';

%de-normalize
for idx = 1:640 %each channel
    sig_ori(:,idx) = sig_ori(:,idx) .* sigma_base_multi_1(idx) + mu_base_multi_1(idx);
    sig_oriNF(:,idx) = sig_oriNF(:,idx) .* sigma_base_multi_1(idx) + mu_base_multi_1(idx);
end

%take average of 20 samples in state
for idx = 2:20
    sig_ori(:,1:32) = sig_ori(:,1:32) + sig_ori(:,(1:32) + 32*(idx-1));
    sig_oriNF(:,1:32) = sig_oriNF(:,1:32) + sig_oriNF(:,(1:32) + 32*(idx-1));

end
sig_ori = sig_ori(:,1:32) / 20;
sig_oriNF = sig_oriNF(:,1:32) / 20;

%plot

%start eeglab (http://sccn.ucsd.edu/eeglab/)
eeglab;

figure('position', [20 10 1000 500]);
set(gcf,'Color',[1 1 1]);

lims = 7.4296;
for state = 1:11 % 6 + 5 states
    subplot(2,6,state);
    if state < 7 %targets/re-paired foils
        topoplot(sig_ori(state,:), chanlocs, 'maplimits', [-lims lims],'style', 'map',...
            'electrodes','off', 'shading', 'interp'); 
    else %new foils
        topoplot(sig_oriNF(state-6,:), chanlocs, 'maplimits', [-lims lims],'style', 'map',...
            'electrodes','off', 'shading', 'interp');
    end
end

  


% Gamma distributions

%calc gammas targets/re-paired foils
tmp = zeros(6,13,8,20); % 6 states x 13 datapoints x 8 conditions x 20 subjects
for subj = 1:20
    for cnd = 1:8
        for st = 1:6
            tmp(st,:,cnd,subj) = gammaN(params(st,1,cnd,subj), params(st,2,cnd,subj),12);
        end
    end
end

gammasTRF = mean(tmp,4); %average subjects
gammasTRF = mean(gammasTRF,3); %average conditions

%calc gammas new foils
tmp = zeros(5,13,2,20); % 5 states x 13 datapoints x 2 conditions x 20 subjects
for subj = 1:20
    for cnd = 1:2
        for st = 1:5
            tmp(st,:,cnd,subj) = gammaN(paramsNF(st,1,cnd,subj), paramsNF(st,2,cnd,subj),12);
        end
    end
end

gammasNF = mean(tmp,4); %average subjects
gammasNF = mean(gammasNF,3); %average conditions


%plot
figure('position', [20 10 800 400]);
set(gcf,'Color',[1 1 1]);

%targets/re-paired foils
subplot(1,2,1,'box', 'on', 'xlim', [-80,1040], 'ylim', [-.05 1.05], 'xtick', 0:160:960);
set(gca,'FontSize',14);
set(gca,'FontName','Helvetica');
xlabel('Duration (ms)','FontSize',15);
ylabel('Proportion','FontSize',15);
line(0:80:960,gammasTRF','linestyle', '-','marker','o','linewidth',1.0,'MarkerSize',8);
legend('State 1', 'State 2', 'State 3', 'State 4', 'State 5', 'State 6','location','northeast');
legend('boxoff');

%new foils
subplot(1,2,2,'box', 'on', 'xlim', [-80,1040], 'ylim', [-.05 1.05],  'xtick', 0:160:960);
set(gca,'FontSize',14);
set(gca,'FontName','Helvetica');
xlabel('Duration (ms)','FontSize',15);
ylabel('Proportion','FontSize',15);
line(0:80:960,gammasNF','linestyle', '-','marker','o','linewidth',1.0,'MarkerSize',8);
legend('State 1', 'State 2', 'State 3', 'State 5', 'State 6','location','northeast');
legend('boxoff');




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure 6: State durations
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('solution_6_targets_repaired_foils.mat');
load('solution_5_new_foils.mat');
ratio = 2 / (1/12.5); %original code assumes 2 second scans, this is the ratio to 80 ms samples

%calculate durations based on gammas - targets/re-paired foils
means = zeros(8, 6, 20);
for i = 1:20
    means(:,:,i) = squeeze(prod(params(:,:,:,i),2))' / ratio * 1000;
end
sds = std(means,0,3)/sqrt(20);
means = mean(means,3);

%calculate durations based on gammas - new foils
meansNF = zeros(2, 5, 20);
for i = 1:20
    meansNF(:,:,i) = squeeze(prod(paramsNF(:,:,:,i),2))' / ratio * 1000;
end
sdsNF = std(meansNF,0,3)/sqrt(20);
meansNF = mean(meansNF,3);



%plot
figure('position', [20 10 700 800]);
set(gcf,'Color',[1 1 1]);
colors = jet(8);
colors = colors([1:3,5,7:8],:);

%bar plot
meansAll = nan(10,6);
meansAll(10:-1:3,:) = means;
meansAll(2:-1:1,[1:3,5:6]) = meansNF;

subplot(2,1,1,'box','on');
set(gca,'FontSize',14);
set(gca,'FontName','Helvetica');
set(gca, 'YTick', 1:10); %10 conditions
h = barh(meansAll,'stacked');
for i=1:6
    set(h(i),'FaceColor',colors(i,:));
end

axis([0 1450 0 11]);
xlabel('Time (ms)','FontSize',15);
ylabel('Condition','FontSize',15);
set(gca, 'YTickLabel', {'New foils - Long','New foils - Short',...
    'Re-paired foils - Fan 2 - Long','Re-paired foils - Fan 2 - Short',...
    'Re-paired foils - Fan 1 - Long','Re-paired foils - Fan 1 - Short',...
    'Targets - Fan 2 - Long','Targets - Fan 2 - Short',...
    'Targets - Fan 1 - Long','Targets - Fan 1 - Short'});

%line plot
meansAll = nan(10,6);
meansAll(1:8,:) = means;
meansAll(9:10,[1:3,5:6]) = meansNF;
sdsAll = nan(10,6);
sdsAll(1:8,:) = sds;
sdsAll(9:10,[1:3,5:6]) = sdsNF;

subplot(2,1,2,'box','on');
axis([0 11 45 405]);
set(gca,'YGrid','on');
set(gca,'FontSize',14);
set(gca,'FontName','Helvetica');
ylabel('Duration (ms)','FontSize',15);

hold on;

for i = 1:size(meansAll,2)
    errorbar(1:size(meansAll,1),meansAll(:,i),sdsAll(:,i),...
        'linewidth', 1.2, 'color', colors(i,:));
end

set(gca, 'XTick', 1:10);
%conditions go in same order as bar graph from top to bottom

legend('State 1', 'State 2', 'State 3', 'State 4', 'State 5', 'State 6','location','northeast');
legend('boxoff');




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Figure 7: EEG differences
% Requires loading data, see above.
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('solution_6_targets_repaired_foils.mat');
load('solution_5_new_foils.mat');
load('chanlocs.mat'); %channel locations


%collect all data - targets, re-paired foils, and new foils
datplot = zeros(20,6,10,640);
for subj = 1:20
    for st = 1:6
        for cnd = 1:10
            if cnd < 9 %targets / rep foils
                datplot(subj,st,cnd,:) = (probpairs(subs==subj & conds == cnd,st)' * act(subs==subj & conds == cnd,2:end)) ./ sum(probpairs(subs==subj & conds == cnd,st));
            else %new foils
                if st < 6 %no 6th state for new foils
                   datplot(subj,st,cnd,:) = (probpairsNF(subsNF==subj & condsNF == cnd - 8, st)' * actNF(subsNF==subj & condsNF == cnd - 8,2:end)) ./ sum(probpairsNF(subsNF==subj & condsNF == cnd-8,st)); 
                end
                
            end
        end
    end
end

%make state 5 new foils state 6 and state 4 new foils state 5
datplot(:,6,9:10,:) = datplot(:,5,9:10,:);
datplot(:,5,9:10,:) = datplot(:,4,9:10,:);
datplot(:,4,9:10,:) = NaN;




%sets definition
%long - short, fan 2 - fan 1, target - rep foil, target - newfoils, rep foil -
%newfoil
sets1 = {[2 4 6 8 10], [3 4 7 8], [1 2 3 4], [1 2 3 4] [5 6 7 8]};
sets2 = {[1 3 5 7 9], [1 2 5 6], [5 6 7 8], [9 10], [9 10]};


%collect differences & plot - ugly code... 

%plot
figure('position', [10 20 1000 700]);
set(gcf,'Color',[1 1 1]);

cnt=0;

for setidx = 1:5
    for st = 1:6
        cnt = cnt + 1;
        if ~(st == 4 && setidx > 3) %new foils don't exist for state 4
            
            %calc difference per subject
            dif = zeros(20,640); %sub x electrodes
            for subj = 1:20
                
                %set 1
                tmp1 = nanmean(squeeze(datplot(subj,st,ismember(1:10, sets1{setidx}),:)));
                
                %recalc to normal vals
                tmp1 = tmp1(:,:) * coeff_base_multi(:,:)';
                for idx = 1:size(tmp1,2)
                    tmp1(:,idx) = tmp1(:,idx) .* sigma_base_multi_1(idx) + mu_base_multi_1(idx);
                end
                
                %set2
                tmp2 = nanmean(squeeze(datplot(subj,st,ismember(1:10, sets2{setidx}),:)));
                
                %recalc to normal vals
                tmp2 = tmp2(:,:) * coeff_base_multi(:,:)';
                for idx = 1:size(tmp2,2)
                    tmp2(:,idx) = tmp2(:,idx) .* sigma_base_multi_1(idx) + mu_base_multi_1(idx);
                end
                
                %difference
                dif(subj,:) = tmp1 - tmp2;
            end
            
            %take average in snapshot
            tmp2 = zeros(20,32,20);
            for idx234 = 1:20
                tmp2(:,:,idx234) = dif(:,(1:32) + 32 * (idx234-1));
            end
            tmp = mean(tmp2,3);
            
            %t-test + FDR
            [~,p,~,stats] = ttest(tmp);
            ts = stats.tstat;
            pfdr = fdr(p');
            ts(pfdr > .01) = 0;
            
            lims = 9.0;
            
            %plot
            subplot(5,6,cnt);
            topoplot(ts, chanlocs, 'maplimits', [-lims lims],'style', 'map',...
                'electrodes','off', 'shading', 'interp');
        end
    end
    
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







